##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################

"""
    News Tag product - BBC News File Decoder
    
File Format:

<DIV ID="stories">
  <DIV ID="1">
    <DIV ID=_NAME_>_VALUE_</DIV>
    <DIV ID=_NAME_>_VALUE_</DIV>
    <DIV ID=_NAME_>_VALUE_</DIV>
  </DIV>
  <DIV ID="1">
    <DIV ID=_NAME_>_VALUE_</DIV>
    <DIV ID=_NAME_>_VALUE_</DIV>
    <DIV ID=_NAME_>_VALUE_</DIV>
  </DIV>
</DIV>
<DIV ID="properties">
  <DIV ID="itemcount">2</DIV>
</DIV>
"""
__version__ = "1.0"

import sgmllib
import BaseParser

class BBC(BaseParser.BaseParser, sgmllib.SGMLParser):
    """ """
    name = 'BBC'
    server = 'news.bbc.co.uk'
    sources = {
        'News': '/ticker/ticker_3_-1_-1_252.inc',
        'Sport': '/ticker/ticker_3_-1_-1_2513.inc',
    }

    def __init__(self, server=None):
        """ """
        self.reset_parser(server)
        self.outsideData = ''         
        self.inStories = 0
        self.currentStory = 0
        self.currentStoryDict = {}
        self.currentItem = ''
        self.currentItemValue = ''
        sgmllib.SGMLParser.__init__(self, 0)

    def feed(self, data):
        """ Feed the data into the parser data stack """
        sgmllib.SGMLParser.feed(self, data)
    
    def start_div(self, attrs):
        """ Work out what we are playing at """
        if attrs:
            for k, v in attrs:
                if k=='id':
                    myValue = v
            if self.inStories:
                if self.currentStory:
                    if self.currentItem:
                        raise "News data structure too deep", ""
                    else:
                        self.currentItem = myValue
                else:
                    self.currentStory = myValue
            else:
                if myValue=='stories':
                    self.inStories = 1
                    self.Stories = []
        
    def handle_data(self, data):
        """ Store the block value against the current div id """
        if self.inStories:
            if self.currentStory:
                if self.currentItem:
                    self.currentItemValue = "%s%s" % (self.currentItemValue, data)
        else:
            self.outsideData = "%s%s" % (self.outsideData, data)
    
    def end_div(self):
        """ Pop the last div off the stack """
        if self.inStories:
            if self.currentStory:
                if self.currentItem:
                    if self.currentItemValue:
                        self.currentStoryDict[self.currentItem] = self.currentItemValue
                    self.currentItem = ''
                    self.currentItemValue = ''
                    return
                self.Stories.append(self.currentStoryDict)
                self.currentStory = 0
                self.currentStoryDict = {}
                return
            self.inStories = 0

    def close(self):
        """ Close the input stream and process the received data """
        sgmllib.SGMLParser.close(self)
        for c in self.Stories:
            if c['SiteLink'][:5] != 'http:':
                c['SiteLink'] = "http://%s%s" % (self.server, c['SiteLink'])
            if c['UrlLink'][:5] != 'http:':
                c['UrlLink'] = "http://%s%s" % (self.server, c['UrlLink'])
            c['url'] = c['UrlLink']
            try:
                c['headline'] = c['LinksTo']
            except:
                c['headline'] = "BBC News Lead Story"
            c['body'] = c['Summary']
