##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################
"""
    News Tag Product
    Adds a new command 'dtml-news' to download news and then render a block
    with the returned data. Supports 'dtml-else' to catch errors cleanly.
    
    Functions like 'dtml-in', in that the returned data is added to the
    environment. The following will exist, though individual data-parsers
    may add others:
        url         # The link for this item
        headline    # The Headline for this item
        body        # The teaser for this item
"""
__version__ = "1.0"

from DocumentTemplate.DT_Util import *
from DocumentTemplate.DT_String import String
import httplib

# Load the BaseParser which includes the (initially empty) parser list.
import BaseParser

# Load parsers. This list can be as long as you like.
import BBC       # BBC News and Sport
import ECT       # E-Commerce Times
import ETel      # e.Tel
import Aviation  # Aviation News
import StarTrek  # Star Trek
import WebTicker # WebTicker

def NTinit():
    BaseParser.ParserList['BBC'] = BBC.BBC
    BaseParser.ParserList['ECT'] = ECT.ECT
    BaseParser.ParserList['ETel'] = ETel.ETel
    BaseParser.ParserList['Aviation'] = Aviation.Aviation
    BaseParser.ParserList['StarTrek'] = StarTrek.StarTrek
    BaseParser.ParserList['WebTicker'] = WebTicker.WebTicker

class NewsServer:
    """ One news server attachment """

    def __init__(self, blocks, type='', source='', localfile=''):
        """ Attach parser to datasource """
        self.blocks = blocks
# Process arguments.
        if type:
            if type in BaseParser.ParserList.keys():
                self.type = type
                self.parser = BaseParser.ParserList[self.type]
            else:
                raise ParseError, ("Invalid 'type' parameter, valid values: '%s'" % join(BaseParser.ParserList.keys(), "','"), "")
        else:
            raise ParseError, ("Missing 'type' parameter, valid values: '%s'" % join(BaseParser.ParserList.keys(), "','"), "")
        if localfile:
            self.localfile = localfile
        else:
            self.localfile = None
            if source:
                if self.parser.sources.has_key(source):
                    self.source = self.parser.sources[source]
                else:
                    raise ParseError, ("Invalid 'source' parameter for 'type=%s', valid values: '%s'" % (self.type, join(self.parser.sources.keys(), "','")), "")
            else:
                if self.parser.sources.has_key('default'):
                    self.source = self.parser.sources['default']
                else:
                    raise ParseError, ("Missing 'source' parameter and no default for 'type=%s', valid values: '%s'" % (self.type, join(self.parser.sources.keys(), "','")), "")

    def render(self, md):
        """
            Get the data then call the selected parser which should return
            a list of dictionaries containing at least the following keys:
                Headline
                URL
                Body
            Then, for each dictionary,
                push into namespace
                render block
                pop from namespace
        """
        res = []
        parser = self.parser()
        if self.localfile:
            parser.feed("%s" % md.getitem(self.localfile, 1))
        else:
            try:
                h = httplib.HTTP(parser.server)
            except:
                raise "Failed to connect to server %s" % parser.server, ""
            h.putrequest('GET', self.source)
            h.putheader('Accept', 'text/plain')
            h.putheader('Accept', 'text/html')
            h.putheader('Accept', 'text/*')
            h.endheaders()
            errcode, errmsg, headers = h.getreply()
            if errcode == 200:
                parser.feed("%s" % h.getfile().read())
            elif errcode == 301:
                raise "HTTP error:%d, message:%s, url:[http://%s%s]<hr><pre>%s</pre>" % (errcode, errmsg, self.parser.server, self.source, headers), ""
            else:
                raise "HTTP error:%d, message:%s, url:[http://%s%s]" % (errcode, errmsg, self.parser.server, self.source), ""
        parser.close()
# Run the section once for each news article loaded.
        if len(parser.Stories):
            for c in parser.Stories:
                c['sequence-item'] = c
                md._push(c)
                blk = render_blocks(self.blocks, md)
                res.append(blk)
                md._pop()
        else:
            return ''
        return join(res, '')

class NewsTag:
    """ dtml-news tag """
    blockContinuations=('news', 'else',)
    name='news'

    def __init__(self, blocks):
        """ Verify parameters etc... """
        NTinit()
        self.sections = []
        self.elses = None
# Get block data.
        for b in blocks:
            tname, args, section = b
            if tname == 'news':
                type = ''
                source = ''
                localfile = ''
                args = parse_params(args, type='', source='', localfile='')
                if args.has_key('type'):
                    type = args['type']
                if args.has_key('source'):
                    source = args['source']
                if args.has_key('localfile'):
                    localfile = args['localfile']
                self.sections.append(NewsServer(section.blocks, type, source, localfile))
            elif tname == 'else':
                if self.elses:
                    raise ParseError, ('too many else blocks', 'news')
                else:
                    self.elses = section.blocks

    def render(self, md):
        """
            Render the sections in order.
            If all sections are empty then render the ELSE if there is one.
        """
        results=[]
        for c in self.sections:
            s=c.render(md)
            if s:
                results.append(s)
        if not results:
            if self.elses:
                return render_blocks(self.elses, md)
        return join(results, '')

    __call__ = render

String.commands['news'] = NewsTag
